#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <usb.h>
#include <arpa/inet.h>

#include "usbtmc.h"

//Helper-routine: Convert a little-endian 4-byte word to an int
static void int2chars(unsigned char *buff,unsigned int a) {
	buff[3]=(a>>24)&0xff;
	buff[2]=(a>>16)&0xff;
	buff[1]=(a>>8)&0xff;
	buff[0]=(a)&0xff;
}

//Helper-routine: Convert an int to  little-endian 4-byte word
unsigned int chars2int(unsigned char *buff) {
	unsigned int a;
	a=buff[3]<<24;
	a+=buff[2]<<16;
	a+=buff[1]<<8;
	a+=buff[0];
	return a;
}

//This routine locates a scope by VID/PID and returns an opened handle to it.
static usb_dev_handle *usbtmc_find_scope() {
	struct usb_bus *bus;
	struct usb_device *dev=NULL;
	usb_find_busses();
	usb_find_devices();
	for (bus=usb_busses; bus; bus=bus->next) {
		for (dev=bus->devices; dev; dev=dev->next) {
			//fprintf(stderr,"Prod/dev: %04X:%04X\n",dev->descriptor.idVendor,dev->descriptor.idProduct);
			if (dev->descriptor.idVendor==0x400 && dev->descriptor.idProduct==0x5dc) {
				return usb_open(dev);
			}
		}
	}
	return NULL;
}

//Send a scpi-command to the scope. The response goes into the buffer
//called resp, with a size of resplen. If resp==NULL, no response
//is requested.
int usbtmc_sendscpi(usb_dev_handle *dev, char* cmd, 
		unsigned char *resp, int resplen) {
	unsigned char *buff;
	int len,r,i;
	int cmdlen = strlen(cmd);
	static unsigned char seq=0;


	buff=malloc(0x40);
	seq++;
	buff[0]=1; //func
	buff[1]=seq; buff[2]=~seq; //nseq
	buff[3]=0;
	int2chars(buff+4, cmdlen);
	buff[8]=1;
	buff[9]=0x37;
	buff[10]=0x39;
	buff[11]=0x39;
	//fprintf(stderr,"Writing header len=%d\n", cmdlen);
	//printb(buff,12);
	r=usb_bulk_write(dev, 1, (char*)buff, 12, 1000);
	//fprintf(stderr,"%i bytes written. Writing cmd\n",r);
	//printb(cmd, cmdlen);
	r=usb_bulk_write(dev, 1, cmd, cmdlen, 1000);
	//fprintf(stderr,"%i bytes written.\n",r);
	if (resp != NULL && resplen != 0) {
		//send read command
		buff[0]=2; //func
		seq++;
		buff[1]=seq; buff[2]=~seq; //nseq
		int2chars(buff+4,0x40);
		buff[8]=1;
		buff[9]=0xA;
		buff[10]=0;
		buff[11]=0;
		//fprintf(stderr,"Writing resp req header\n");
		//printb(buff,12);
		r=usb_bulk_write(dev, 1, (char*)buff, 12, 1000);
		//fprintf(stderr,"%i bytes written. Reading response hdr\n",r);
		r=usb_bulk_read(dev, 2, (char*)buff, 0x40, 1000);
		//printb(buff,r);
		len=chars2int(buff+4);
		//fprintf(stderr,"%i bytes read. Resplen=%i\n",r,len);
		for (i=0; i<(r-12); i++) {
			if (i<resplen) resp[i] = buff[i+12];
		}
		//printb(resp,r-12);
		if (len > 0x40-12) {
			//fprintf(stderr," Reading response:\n");
			if (resplen<len) len=resplen;
			r=usb_bulk_read(dev, 2, (char*)resp+(0x40-12), len-(0x40-12),1000);
			//fprintf(stderr,"%i bytes read, wanted %i.\n", r, len-(0x40-12));
			return r+(0x40-12);
		}
		return len;
	}
	return 0;
}

//Initialize the scope.
usb_dev_handle* usbtmc_initscope(void) {
	int r;
	unsigned char buff[10];
	usb_dev_handle *dev;

	//Init libusb
	usb_init();
	//Locate and open the scope
	dev = usbtmc_find_scope();
	if (!dev) {
		printf("No scope found.\n");
		exit(1);
	} else {
		printf("Scope found.\n");
	}
	usb_claim_interface(dev,0);
	//The following code isn't really necessary, the program works
	//OK without it too.
	r=usb_control_msg(dev, 0xC8, 9, 0, 0, (char*)buff, 4, 1000);
	if (r < 0) {
		fprintf (stderr, "Error %d sending init message: %s\n", 
				r, strerror (-r));
		fprintf (stderr, "Do you have permission on the USB device?\n");
		exit (1);
	}
	if (chars2int(buff)!=0x40004dc) {
		fprintf(stderr,"Init: buff[%i]=%x\n",r,chars2int(buff));
	}
	return dev;
}
